@@ -21,9 +21,29 @@ def _strip_styles(text):
 
 
 def ljust(text, length):
+    """Like str.ljust() but ignore all ANSI controlling characters."""
     return text + " " * (length - len(_strip_styles(text)))
 
 
+class DummySpinner:
+    """A dummy spinner class implementing needed interfaces.
+    But only display text onto screen.
+    """
+
+    def start(self, text: str):
+        stream.echo(text)
+
+    succeed = fail = stop_and_persist = start
+
+    text = property(lambda self: "", start)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        pass
+
+
 class IOStream:
     NORMAL = 0
     DETAIL = 1
@@ -33,7 +53,10 @@ class IOStream:
         self.verbosity = verbosity
         self._disable_colors = disable_colors
         self._indent = ""
-        self.logger = None
+        logger = logging.getLogger(__name__)
+        logger.setLevel(logging.DEBUG)
+        logger.addHandler(logging.NullHandler())
+        self.logger = logger
 
         for color in COLORS:
             setattr(self, color, functools.partial(self._style, fg=color))
@@ -89,18 +112,16 @@ class IOStream:
     def logging(self, type_: str = "install"):
         file_name = mktemp(".log", f"pdm-{type_}-")
 
-        logger = logging.getLogger(__name__)
-        logger.setLevel(logging.DEBUG)
+        logger = self.logger
         if self.verbosity >= self.DETAIL:
             handler = logging.StreamHandler()
         else:
             handler = logging.FileHandler(file_name, encoding="utf-8")
         handler.setLevel(logging.DEBUG)
-        logger.addHandler(handler)
+        logger.handlers[1:] = [handler]
         pip_logger = logging.getLogger("pip.subprocessor")
         pip_logger.handlers[:] = [handler]
         try:
-            self.logger = logger
             yield logger
         except Exception:
             if self.verbosity < self.DETAIL:
@@ -114,8 +135,12 @@ class IOStream:
 
     @contextlib.contextmanager
     def open_spinner(self, title: str, spinner: str = "dots"):
-        with halo.Halo(title, spinner=spinner) as spin:
-            yield spin
+        if self.verbosity >= self.DETAIL:
+            bar = DummySpinner()
+        else:
+            bar = halo.Halo(title, spinner=spinner)
+        with bar as bar:
+            yield bar
 
 
 stream = IOStream()
