@@ -56,7 +56,14 @@ def get_abi_tag(python_version):
     A replacement for pip._internal.models.pep425tags:get_abi_tag()
     """
     try:
-        from wheel.pep425tags import get_abbr_impl, get_config_var, get_flag
+        from wheel.pep425tags import get_abbr_impl, get_config_var
+        from wheel.pep425tags import get_flag as _get_flag
+
+        def get_flag(var, fallback, expected=True, warn=True):
+            return _get_flag(
+                var, fallback=lambda: fallback, expected=expected, warn=warn
+            )
+
     except ModuleNotFoundError:
         from packaging.tags import interpreter_name as get_abbr_impl
         from wheel.bdist_wheel import get_config_var, get_flag
@@ -70,15 +77,15 @@ def get_abi_tag(python_version):
         m = ""
         u = ""
         is_cpython = impl == "cp"
-        if get_flag("Py_DEBUG", lambda: hasattr(sys, "gettotalrefcount"), warn=False):
+        if get_flag("Py_DEBUG", hasattr(sys, "gettotalrefcount"), warn=False):
             d = "d"
         if python_version < (3, 8) and get_flag(
-            "WITH_PYMALLOC", lambda: is_cpython, warn=False
+            "WITH_PYMALLOC", is_cpython, warn=False
         ):
             m = "m"
         if python_version < (3, 3) and get_flag(
             "Py_UNICODE_SIZE",
-            lambda: sys.maxunicode == 0x10FFFF,
+            sys.maxunicode == 0x10FFFF,
             expected=4,
             warn=False,
         ):
@@ -298,7 +305,7 @@ def _wheel_support_index_min(self, tags=None):
 
 
 @contextmanager
-def allow_all_wheels():
+def allow_all_wheels(enable: bool = True):
     """Monkey patch pip.Wheel to allow all wheels
 
     The usual checks against platforms and Python versions are ignored to allow
@@ -308,6 +315,10 @@ def allow_all_wheels():
     """
     from pip._internal.models.wheel import Wheel as PipWheel
 
+    if not enable:
+        yield
+        return
+
     original_wheel_supported = PipWheel.supported
     original_support_index_min = PipWheel.support_index_min
 
@@ -349,15 +360,25 @@ def get_python_version(executable, as_string=False):
     return ".".join(map(str, result))
 
 
-@functools.lru_cache()
-def get_sys_config_paths(executable: str) -> Dict[str, str]:
+def get_sys_config_paths(executable: str, vars=None) -> Dict[str, str]:
     """Return the sys_config.get_paths() result for the python interpreter"""
-    args = [
-        executable,
-        "-c",
-        "import sysconfig,json;print(json.dumps(sysconfig.get_paths()))",
-    ]
-    return json.loads(subprocess.check_output(args))
+    if not vars:
+        args = [
+            executable,
+            "-c",
+            "import sysconfig,json;print(json.dumps(sysconfig.get_paths()))",
+        ]
+        return json.loads(subprocess.check_output(args))
+    else:
+        env = os.environ.copy()
+        env.update(SYSCONFIG_VARS=json.dumps(vars))
+        args = [
+            executable,
+            "-c",
+            "import os,sysconfig,json;print(json.dumps(sysconfig."
+            "get_paths(vars=json.loads(os.getenv('SYSCONFIG_VARS')))))",
+        ]
+        return json.loads(subprocess.check_output(args, env=env))
 
 
 def get_pep508_environment(executable: str) -> Dict[str, Any]:
@@ -508,13 +529,14 @@ def highest_version(versions: List[str]) -> str:
 
 
 def populate_link(
-    finder: PackageFinder, ireq: InstallRequirement, upgrade: bool = False
+    finder: PackageFinder,
+    ireq: InstallRequirement,
+    upgrade: bool = False,
 ):
     """Populate ireq's link attribute"""
-    if ireq.link:
-        return
-    link = finder.find_requirement(ireq, upgrade)
-    if not link:
-        return
-    link = getattr(link, "link", link)
-    ireq.link = link
+    if not ireq.link:
+        link = finder.find_requirement(ireq, upgrade)
+        if not link:
+            return
+        link = getattr(link, "link", link)
+        ireq.link = link
